/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CEConsumer.hh: header file for a C++ providing consumer APIs

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CEConsumer.h,v 1.37 2007/12/14 11:02:37 adorigo Exp $
*/
#ifndef __GLITE_CE_MONITOR_CLIENT_CECONSUMER__
#define __GLITE_CE_MONITOR_CLIENT_CECONSUMER__

#define SOAP_RECV_TIMEOUT 60
#define SOAP_SEND_TIMEOUT 60

#include "glite/ce/monitor-client-api-c/monitor_consumer_CEMON_CONSUMERStub.h"
#include "glite/ce/monitor-client-api-c/Topic.h"
#include <string>
#include <vector>

extern "C" {
#undef IOV_MAX // this is very ugly, but the only way to silent a warning concerning
               // the redefinition of IOV_MAX
#include "glite/security/glite_gsplugin.h"	
}

namespace glite {
  namespace ce {
    namespace monitor_client_api {
      namespace soap_proxy {

class CEConsumer {

private:
  int localPort;
  int localSocket;
  int clientSocket;
  std::string errorMessage;
  std::string errorCode;
  struct soap *SOAP;
  std::string clientIP;
  std::string clientName;
  time_t EventTimestamp;
  Topic* topic;
  int messageSize;
  int EventID;
  std::string EventProducer;
  std::vector<std::string> Messages;
  int messageCounter;
  std::vector<monitortypes__Event> Events;
  bool authn;
  std::string client_dn;
  
protected:
  CEConsumer(const CEConsumer&) {}
  CEConsumer operator=(const CEConsumer&) { return *this; }
  std::string getPeerName( void );

private:
  void setResponse(int, int, time_t, std::string, std::vector<std::string>*);
  void setError(void);
  glite_gsplugin_Context	ctx;
  int alen;// = sizeof(a);
  struct sockaddr_in peerAddress;

public:
  CEConsumer(const int& port, const char* cert=NULL, const char* key=NULL);
  virtual ~CEConsumer();

  bool bind(void);
  bool accept(void);
  void reset(void);
  std::string getClientIP() const { return clientIP; }
  std::string getClientName() const { return clientName; }
  int getClientSocket() const { return clientSocket; }
  int getLocalPort() const { return localPort; }
  int getLocalSocket() const { return localSocket; }
  bool serve(void);
  const char* getNextEventMessage();
  Topic* getEventTopic() const { return topic; }
  int getEventID() const { return EventID; }
  int getMessageSize() const { return messageSize; }
  std::string getEventProducer() const { return EventProducer; }
  std::string getErrorMessage() const { return errorMessage; }
  std::string getErrorCode() const { return errorCode; }
  void setRecvTimeout(const int& t) { SOAP->recv_timeout = t; }
  void setSendTimeout(const int& t) { SOAP->send_timeout = t; }
  const std::vector<monitortypes__Event>& getEvents( void ) const { return Events; }
  const std::string getClientDN( void ) const { return client_dn; }
};
}
}
}
}
#endif
