/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CEException: a C++ Exception related to any CE-generated fault
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CEException.h,v 1.16 2007/12/14 11:02:37 adorigo Exp $
*/

#ifndef __GLITE_CE_MONITOR_CLIENT_CEEXCEPTION_H__
#define __GLITE_CE_MONITOR_CLIENT_CEEXCEPTION_H__

#include <string>
#include <time.h>
#include <sstream>
#include <exception>

namespace glite {
  namespace ce {
    namespace monitor_client_api {
      namespace cemon_exceptions {
	
	class CEException : public std::exception
	  {
	    
	  private:
	    std::string MethodName;
	    std::string ErrorCode;
	    std::string Description;
	    std::string FaultCause;
	    time_t Timestamp;
	    
	  public:
	    
	    /*   enum { */
	    /*     Subscription = 23, */
	    /*     TopicNotSupported = 24, */
	    /*     DialectNotSupported = 25, */
	    /*     Base = 26, */
	    /*     Generic = 27, */
	    /*     Authentication = 28, */
	    /*     Authorization = 29 */
	    /*   }; */
	    
	    CEException(const std::string& methodname, 
			const time_t& tstamp, 
			const std::string& error_code, 
			const std::string& description, 
			const std::string& fault_cause) : 
	      MethodName(methodname), 
	      ErrorCode(error_code), 
	      Description(description), 
	      FaultCause(fault_cause), 
	      Timestamp(tstamp) { }
	    
	    CEException() {}
	    virtual ~CEException() throw() {}
	    
	    const char* what() const throw() {
	      std::ostringstream os("");
	      os << "MethodName=["<<MethodName<<"] Timestamp=["<<
		Timestamp<<"] Description=["<<Description<<"] FaultCause=["<<
		FaultCause<<"] ErrorCode=["<<ErrorCode<<"]"<<EOF;
	      return os.str().c_str();
	    }
	    inline std::string getMethod() { return MethodName; }
	    inline std::string getErrorCode() { return ErrorCode; }
	    inline std::string getDescription() { return Description; }
	    inline std::string getFaultCause() { return FaultCause; }
	    inline time_t      getTimeStamp() { return Timestamp; }
	  };
      }
    }
  }
}

#endif
